@@ -4,23 +4,23 @@ __docformat__ = 'restructuredtext'
 __author__ = "Stephen J. Mildenhall"
 __copyright__ = "Copyright 2018-2022, Convex Risk LLC"
 __license__ = "BSD 3-Clause New License"
-__version__ = "0.9.2"
+__version__ = "0.9.3"
 __email__ = "steve@convexrisk.com"
 __status__ = "alpha"
 
 # imports
-from .param import hack_make_lines_from_csv
-from .underwriter import Underwriter
+from .underwriter import Underwriter, build
 from .port import Portfolio
 from .distr import Frequency, Severity, Aggregate, CarefulInverse
 from .spectral import Distortion
 from .utils import get_fmts, tidy_agg_program, ft, \
     ift, sln_fit, sgamma_fit, estimate_agg_percentile, \
     axiter_factory, AxisManager, lognorm_lev, html_title, \
-    sensible_jump, suptitle_and_tight, insurability_triangle, \
-    read_log, MomentAggregator, MomentWrangler, xsden_to_meancv, \
+    sensible_jump, suptitle_and_tight, \
+    MomentAggregator, MomentWrangler, xsden_to_meancv, \
     frequency_examples, Answer, log_test, subsets, \
-    start_timer, report_time, stop_timer, round_bucket
+    start_timer, report_time, stop_timer, round_bucket, \
+    make_ceder_netter
 from .parser import UnderwritingLexer, UnderwritingParser
 
 
