@@ -11,6 +11,7 @@ import itertools
 from scipy.special import kv
 from scipy.optimize import broyden2, newton_krylov
 from scipy.optimize.nonlin import NoConvergence
+from scipy.interpolate import interp1d
 from io import StringIO
 import re
 from pathlib import Path
@@ -52,7 +53,7 @@ logger.setLevel(logging.DEBUG)
 # kick off message
 logger.debug('aggregate_project.__init__ | New Aggregate Session started')
 
-
+# TODO take out timer stuff
 last_time = first_time = 0
 timer_active = False
 
@@ -261,28 +262,78 @@ def round_bucket(bs):
 
     if bs < 1 find the smallest power of two greater than 1/bs
 
-    """
+    Test cases:
 
-    if bs >= 1:
-        # rounded bs
+        test_cases = [1, 1.1, 2, 2.5, 4, 5, 5.5, 8.7, 9.9, 10, 13, 15, 20, 50, 100, 99, 101, 200, 250, 400, 457,
+                        500, 750, 1000, 2412, 12323, 57000, 119000, 1e6, 1e9, 1e12, 1e15, 1e18, 1e21]
+        for i in test_cases:
+            print(i, round_bucket(i))
+        for i in test_cases:
+            print(1/i, round_bucket(1/i))
+
+    """
+    if bs == 1:
+        return bs
+
+    if bs > 1:
+        # rounded bs, to an integer
         rbs = np.round(bs, 0)
         if rbs == 1:
             return 2.0
+        elif rbs == 2:
+            return 2
         elif rbs <= 5:
             return 5.0
         elif rbs <= 10:
             return 10.0
         else:
-            return 10 * round_bucket(bs / 10)
+            rbs = np.round(bs, -int(np.log(bs) / np.log(10)))
+            if rbs < bs:
+                rbs *= 2
+            return rbs
 
     if bs < 1:
         # inverse bs
-        bsi = 1 / bs
-        nbs = 1
-        while nbs < bsi:
-            nbs <<= 1
-        nbs >>= 1
-        return 1. / nbs
+        # originally
+        # bsi = 1 / bs
+        # nbs = 1
+        # while nbs < bsi:
+        #     nbs <<= 1
+        # nbs >>= 1
+        # return 1. / nbs
+        # same answer but ? clearer and slightly quicker
+        x = 1. / bs
+        x = bin(int(x))
+        x = '0b1' + "0" * (len(x) -3)
+        x = int(x[2:], 2)
+        return 1./ x
+
+def make_ceder_netter(reins_list, debug=False):
+    """
+    Build the netter and ceder functions. It is applied to occ_reins and agg_reins,
+    so should be stand-alone.
+    """
+    h = 0
+    base = 0
+    xs = [0]
+    ys = [0]
+    for (p, y, a) in reins_list:
+        if a > base:
+            xs.append(a)
+            ys.append(h)
+        h += p
+        xs.append(a + y)
+        ys.append(h)
+        base += (a + y)
+    xs.append(np.inf)
+    ys.append(h)
+    ceder = interp1d(xs, ys)
+    netter = lambda x: x - ceder(x)
+    if debug:
+        return ceder, netter, xs, ys
+    else:
+        return ceder, netter
+
 
 # axis management
 def axiter_factory(axiter, n, figsize=None, height=2, aspect=1, nr=5):
@@ -543,82 +594,7 @@ def suptitle_and_tight(title, **kwargs):
     :return:
     """
     plt.suptitle(title, **kwargs)
-    plt.tight_layout(rect=[0, 0, 1, 0.96])
-
-
-# general nonsense
-def insurability_triangle(figsize=(10, 3)):
-    """
-    Illustrate the insurability triangle...
-
-    ::
-
-        λ = A / L
-        ROE = (P-L) / (A - P) = (P/L - 1) / (A/L - P/L)
-            = (1/LR - 1) / (λ - 1/LR) = (1 - LR) / (λLR - 1)
-
-    Hence
-
-    ::
-
-        δ = ROE / (1 + ROE) = (1 - LR) / [LR (λ - 1)]
-
-    :return:
-    """
-    f, axs = plt.subplots(1, 3, figsize=figsize)
-    it = iter(axs.flatten())
-    λs = [1.5, 2, 3, 5, 10, 25, 50, 100]
-    # up to user to manage colors
-    # sns.set_palette(sns.cubehelix_palette(len(λs)))
-
-    LR = np.linspace(0, 1, 101)
-    plt.sca(next(it))
-    for λ in λs:
-        δ = (1 - LR) / LR / (λ - 1)
-        plt.plot(LR, δ, label=f'λ={λ}')
-    plt.legend()
-    plt.ylim(0, 1)
-    plt.xlabel('Loss Ratio')
-    plt.ylabel('δ Investor Discount Rate')
-    plt.title('Profitability vs. Loss Ratio \nBy PML to EL Ratio')
-
-    LR = np.linspace(0, 1, 101)
-    plt.sca(next(it))
-    for λ in λs:
-        levg = np.where(λ * LR > 1, 1 / (λ * LR - 1), 4)  # hide the discontinuity
-        plt.plot(LR, levg, label=f'λ={λ}')
-    # plt.legend()
-    plt.ylim(0, 3)
-    plt.xlabel('Loss Ratio')
-    plt.ylabel('Premium to Surplus Ratio')
-    plt.title('Premium to Surplus Ratio vs. Loss Ratio \nBy PML to EL Ratio')
-
-    δ = np.linspace(0.0, 0.3, 301)
-    plt.sca(next(it))
-    for λ in λs:
-        LR = 1 / (1 + δ * (λ - 1))
-        plt.plot(δ, LR, label=f'λ={λ}')
-    # plt.legend()
-    plt.ylim(0, 1)
-    plt.ylabel('Loss Ratio')
-    plt.xlabel('Investor Discount Rate')
-    plt.title('Loss Ratio vs. Investor Discount Rate \nBy PML to EL Ratio')
-
-
-def read_log():
-    """
-    read and return the log file
-
-    :return:
-    """
-
-    df = pd.read_csv(LOGFILE, sep='|', header=0, names=['datetime', 'context', 'type', 'routine', 'log'],
-                     parse_dates=[0])
-    for c in df.select_dtypes(object):
-        df[c] = df[c].str.strip()
-    df = df.dropna()
-    df = df.set_index('datetime')
-    return df
+    # plt.tight_layout(rect=[0, 0, 1, 0.96])
 
 
 class MomentAggregator(object):
@@ -984,76 +960,6 @@ class MomentWrangler(object):
         self._factorial = (ex1, ex2 - ex1, ex3 - 3 * ex2 + 2 * ex1)
 
 
-# class qd(object):
-#     """
-#     quick display for dictionaries and Pandas dataframes, with some sensible number defaults
-#     experimental
-#
-#     """
-#
-#     # Set CSS properties for th elements in dataframe
-#     th_props = [
-#         ('font-size', '11px'),
-#         ('text-align', 'center'),
-#         ('font-weight', 'bold'),
-#         ('color', '#6d6d6d'),
-#         ('background-color', '#f7f7f9')
-#     ]
-#
-#     # Set CSS properties for td elements in dataframe
-#     td_props = [
-#         ('font-size', '10px'),
-#         ('text-align', 'left')
-#     ]
-#
-#     # Set table styles
-#     styles = [
-#         dict(selector="th", props=th_props),
-#         dict(selector="td", props=td_props)
-#     ]
-#
-#     cm = sns.light_palette("green", as_cmap=True)
-#
-#     def __init__(self, d):
-#         self.x = d
-#
-#     def _repr_html_(self):
-#         if isinstance(self.x, dict):
-#             return pd.DataFrame(self.x, index=[len(self.x)])._repr_html_()
-#         if isinstance(self.x, list) or isinstance(self.x, tuple) and len(self.x) == 2:
-#             if isinstance(self.x[1], dict):
-#                 return f'<h2>{self.x[0]}</h2><br>' + qd(self.x[1])._repr_html_()
-#         elif isinstance(self.x, pd.DataFrame):
-#             # do a bit of styling
-#             num_cols = self.x.select_dtypes(np.number).columns
-#             fmt = {}
-#             for a, b in zip(self.x.columns, self.x.dtypes):
-#                 if np.issubdtype(b, np.number):
-#                     m, s = self.x[a].agg([np.mean, np.std])
-#                     x = np.abs(m) + 3 * s
-#                     if abs(x) > self.x[a].max():
-#                         x = self.x[a].max()
-#                     if x < 10:
-#                         fmt[a] = '{:7.3f}'
-#                     elif x < 1000:
-#                         fmt[a] = '{:7.1f}'
-#                     elif x < 10e6:
-#                         fmt[a] = '{:12,.1f}'
-#                     else:
-#                         fmt[a] = '{:12.3e}'
-#                 else:
-#                     fmt[a] = '{:}'
-#
-#             return (self.x.style
-#                     .background_gradient(cmap=cm, subset=num_cols)
-#                     .highlight_max(subset=num_cols)
-#                     #   .set_caption('This is a custom caption.')
-#                     .format(fmt)
-#                     .set_table_styles(styles))._repr_html_()
-#         else:
-#             return repr(self.x)
-
-
 def xsden_to_meancv(xs, den):
     """
     compute mean and cv from xs and density
@@ -1081,6 +987,8 @@ def frequency_examples(n, ν, f, κ, sichel_case, log2, xmax=500, **kwds):
 
     sichel_case = gamma | ig | ''
 
+    Sample call: df, ans = frequency_examples(n=100, ν=0.45, f=0.5, κ=1.25, sichel_case='', log2=16, xmax=2500)
+
     n = E(N) = expected claim count
     ν = CV(mixing) = asymptotic CV of any compound aggregate whose severity has a second moment
     f = proportion of certain claims, 0 <= f < 1, higher f corresponds to greater skewnesss
@@ -1387,13 +1295,14 @@ def frequency_examples(n, ν, f, κ, sichel_case, log2, xmax=500, **kwds):
         # pal = [sns.color_palette("Paired", 7)[i] for i in [all_dist.index(j) for j in vars]]
         df[vars].plot(kind='line', ax=next(axiter)) #, color=pal)
         axiter.ax.set_xlim(0, 4 * n)
-        df[vars].plot(kind='line', logy=True, ax=next(axiter), legend=None, color=pal)
+        df[vars].plot(kind='line', logy=True, ax=next(axiter), legend=None) # , color='virdis')
     axiter.tidy()
     display(ans.unstack())
     return df, ans
 
 
 class Answer(dict):
+    # TODO replace with collections.namedtuple? Or at least, stop using it!
     def __init__(self, **kwargs):
         """
         Generic answer wrapping class with plotting
